/*
** Module   : AbstractChildProcess.java
** Abstract : base implementation for interactive child processes
**
** Copyright (c) 2015, Golden Code Development Corporation.
** ALL RIGHTS RESERVED. Use is subject to license terms.
**
**           Golden Code Development Corporation
**                      CONFIDENTIAL
**
** -#- -I- --Date-- ------------------------------Description----------------------------------
** 001 GES 20150410 First version.
*/

package com.goldencode.p2j.ui.client.chui.driver;

import com.goldencode.p2j.ui.chui.*;
import com.goldencode.p2j.util.*;

/**
 * Provides a common implementation for an interactive child process which is independent from
 * any specific UI modality. 
 */
public abstract class AbstractChildProcess
implements InteractiveChildProcess
{
   /** Core client helpers. */
   protected ThinClient terminal = null;
   
   /** Batch mode flag. */
   protected boolean batch = false;
   
   /** Artificial terminal flag (<code>true</code> if the client has no native terminal). */
   protected boolean artificial = false;
   
   /**
    * Construct an instance.
    *
    * @param    terminal
    *           Core client helpers.
    * @param    batch
    *           Batch mode flag.
    * @param    artificial
    *           Artificial terminal flag (<code>true</code> if the client has no native
    *           terminal).
    */
   public AbstractChildProcess(ThinClient terminal, boolean batch, boolean artificial)
   {
      this.terminal   = terminal;
      this.batch      = batch;
      this.artificial = artificial;
   }
   
   /**
    * Reports if the client is in batch mode.
    *
    * @return   <code>true</code> if in batch mode.
    */
   public boolean inBatchMode()
   {
      return batch;
   }
   
   /**
    * Reports if the client is missing an already initialized native terminal session.
    *
    * @return   <code>true</code> if this is an artificial terminal (there is no native
    *           terminal).
    */
   public boolean isArtificialTerm()
   {
      return artificial;
   }
   
   /**
    * Wait for the user to press a key.
    *
    * @return   The key code of the key pressed to clear the pause or -1 on any failure.
    */
   public int pauseForUser()
   {
      return terminal.suspendedPause();
   }

   /**
    * Temporarily halts the client UI processing (key reading, normal drawing
    * mode)  After entering suspend mode, the current contents of the
    * terminal are cleared.  Use {@link #resume} to return to normal mode.
    */
   public void suspend()
   {
      terminal.suspend();
   }
   
   /**
    * Re-enables client UI processing (key reading, normal drawing mode).
    * Before resuming the current contents of the terminal are cleared.  This
    * is the mechanism to return to normal processing mode after having
    * called {@link #suspend}.
    */
   public void resume()
   {
      terminal.resume();
   }
}
