/*
** Module   : VT100EmulatorChildProcess.java
** Abstract : interactive child process for clients that have an emulated vt100 terminal
**
** Copyright (c) 2015, Golden Code Development Corporation.
** ALL RIGHTS RESERVED. Use is subject to license terms.
**
**           Golden Code Development Corporation
**                      CONFIDENTIAL
**
** -#- -I- --Date-- ------------------------------Description----------------------------------
** 001 GES 20150413 First version.
*/

package com.goldencode.p2j.ui.client.chui.driver;

import com.goldencode.p2j.ui.chui.*;
import com.goldencode.p2j.ui.client.*;
import com.goldencode.p2j.util.*;
import com.goldencode.terminal.*;

/**
 * Provides an implementation for an interactive child process for clients that have a native
 * terminal built-in to the JVM process.
 */
public abstract class VT100EmulatorChildProcess
extends AbstractChildProcess
{
   /** Child process wrapped by a VT100 terminal emulator. */
   protected VT100Emulator emulator = null;
   
   /** Emulator worker thread. */
   protected Thread worker = null;
   
   /**
    * Construct an instance.
    */
   public VT100EmulatorChildProcess()
   {
      super(ThinClient.getInstance(), false, false);
   }
   
   /**
    * Create a shell emulator and execute the given command.
    * 
    * @param    cmdline
    *           The command.
    *
    * @return   A VT100 emulator executing the given command.
    */
   public abstract VT100Emulator createShellEmulator(String[] cmdline);
   
   /**
    * Launch the specified command line as an interactive child process.
    *
    * @param    cmdline
    *           The command line to execute.  The first element must be the program name.
    * @param    silent
    *           <code>false</code> for interactive mode (there the terminal is expected to be
    *           shared by the child process and the user is allowed to directly interact with
    *           the child process.
    *
    * @return   The operating system process ID (pid) of the child process.
    */
   @Override
   public int launch(String[] cmdline, boolean silent)
   {
      emulator = createShellEmulator(cmdline);
            
      // get child process pid.
      int pid = emulator.getPid();
      
      if (pid >= 0)
      {
         // start pseudo terminal, this is the thread that will read output from the child
         // process and write that output to the simulated terminal (displayed to the user)
         worker = new Thread(emulator);
         worker.start();
      }
      
      return pid;
   }
   
   /**
    * Block execution until the interactive child process has exited. This will ensure that
    * any operating system resources are released before returning.
    *
    * @param    pid
    *           The operating system process ID of the child process on which to wait.
    * @param    silent
    *           <code>false</code> for interactive mode (there the terminal is expected to be
    *           shared by the child process and the user is allowed to directly interact with
    *           the child process.
    */
   @Override
   public void waitForExit(int pid, boolean silent)
   throws InterruptedException
   {
      if (emulator == null || worker == null)
      {
         // TODO: log this?
         return;
      }
      
      // wait for interactive process to die
      emulator.waitFor();
      
      // stop the emulator and wait for the worker thread to stop
      emulator.stopRunning();
      worker.join();
   }
   
   /**
    * Asynchronously cleanup any operating system resources when the interactive child process
    * has exited.  This processing should be done an a secondary thread and this method will
    * return immediately after starting that thread.
    *
    * @param    pid
    *           The operating system process ID of the child process on which to wait.
    */
   @Override
   public void asynchCleanup(int pid)
   {
      Thread t1 = new Thread(new Cleaner());
      t1.start();
   }
   
   /**
    * Wait for the user to press a key.
    *
    * @return   The key code of the key pressed to clear the pause or -1 on any failure.
    */
   @Override
   public int pauseForUser()
   {
      Window.resolveWindow().status("Press space bar to continue.");
      return super.pauseForUser();
   }
   
   /**
    * Provides a simple mechanism to wait for the exit of a process on an asynchronous basis
    * (on a new thread dedicated to this wait).
    */
   private class Cleaner
   implements Runnable
   {
      /**
       * Blocks until the child process exits.
       */
      public void run()
      {
         emulator.waitFor();
      }
   }
}
