/*
** Module   : NativeChildProcess.java
** Abstract : interactive child process for clients that have a native terminal
**
** Copyright (c) 2015, Golden Code Development Corporation.
** ALL RIGHTS RESERVED. Use is subject to license terms.
**
**           Golden Code Development Corporation
**                      CONFIDENTIAL
**
** -#- -I- --Date-- ------------------------------Description----------------------------------
** 001 GES 20150410 First version.
*/

package com.goldencode.p2j.ui.client.chui.driver.console;

import com.goldencode.p2j.ui.client.chui.driver.*;
import com.goldencode.p2j.ui.chui.*;
import com.goldencode.p2j.ui.client.driver.*;
import com.goldencode.p2j.util.*;

/**
 * Provides an implementation for an interactive child process for clients that have a native
 * terminal built-in to the JVM process.
 */
public class NativeChildProcess
extends AbstractChildProcess
{
   /** Background mode flag. */
   protected boolean background = false;
   
   /**
    * Construct an instance.
    *
    * @param    driver
    *           The client driver instance in use.
    */
   public NativeChildProcess(ScreenDriver driver)
   {
      super(ThinClient.getInstance(), driver.inBatchMode(), false);
      
      this.background = driver.inBackgroundMode();
   }
   
   /**
    * Launch the specified command line as an interactive child process.
    *
    * @param    cmdline
    *           The command line to execute.  The first element must be the program name.
    * @param    silent
    *           <code>false</code> for interactive mode (there the terminal is expected to be
    *           shared by the child process and the user is allowed to directly interact with
    *           the child process.
    *
    * @return   The operating system process ID (pid) of the child process.
    */
   public int launch(String[] cmdline, boolean silent)
   {
      return LaunchManager.pseudoTerminalLaunch(cmdline, null, null, null, silent, background);
   }
   
   /**
    * Block execution until the interactive child process has exited. This will ensure that
    * any operating system resources are released before returning.
    *
    * @param    pid
    *           The operating system process ID of the child process on which to wait.
    * @param    silent
    *           <code>false</code> for interactive mode (there the terminal is expected to be
    *           shared by the child process and the user is allowed to directly interact with
    *           the child process.
    */
   public void waitForExit(int pid, boolean silent)
   throws InterruptedException
   {
      LaunchManager.pseudoTerminalWait(pid, silent);
   }
   
   /**
    * Asynchronously cleanup any operating system resources when the interactive child process
    * has exited.  This processing should be done an a secondary thread and this method will
    * return immediately after starting that thread.
    *
    * @param    pid
    *           The operating system process ID of the child process on which to wait.
    */
   public void asynchCleanup(int pid)
   {
      Thread t1 = new Thread(new Cleaner(pid));
      t1.start();
   }
   
   /**
    * Provides a simple mechanism to wait for the exit of a process on an
    * asynchronous basis (on a new thread dedicated to this wait).
    */
   private static class Cleaner
   implements Runnable
   {
     /** The child process identifier for which to wait. */
      private int pid = -1;
      
      /**
       * Constructs an instance that has the data needed to block until the
       * child process exits.
       *
       * @param    pid
       *           Identifies the child process for which to wait.
       */
      public Cleaner(int pid)
      {
         this.pid = pid;
      }
      
      /**
       * Blocks until the child process exits.
       */
      public void run()
      {
         // block until the child process exits
         LaunchManager.pseudoTerminalWait(pid);
      }
   }
}
