/*
** Module   : ToolTip.java
** Abstract : GUI tooltip feature implementation.
**
** Copyright (c) 2014-2015, Golden Code Development Corporation.
** ALL RIGHTS RESERVED. Use is subject to license terms.
**
**           Golden Code Development Corporation
**                      CONFIDENTIAL
**
** -#- -I- --Date-- ---------------------------------Description---------------------------------
** 001 EVL 20141201 Initial version.
** 002 EVL 20141209 Fix for tooltip offset.  Separating tooltip into own thread.
** 003 HC  20150323 Removed business logic from config classes.
** 004 GES 20150425 Eliminated AWT dependencies.
*/

package com.goldencode.p2j.ui.client.gui;

import com.goldencode.p2j.ui.*;
import com.goldencode.p2j.ui.client.*;
import com.goldencode.p2j.ui.chui.ThinClient;
import com.goldencode.p2j.ui.client.gui.driver.*;
import com.goldencode.p2j.ui.client.widget.*;

/**
 * The help string showing when mouse become over the GUI widget.  There is no counterpart in
 * character mode interface.
 */
@SuppressWarnings("unchecked")
public class ToolTip
implements Runnable
{
   /** The distance from mouse pointer coordinate to draw tooltip. */
   private static final int ORIGIN_SHIFT = 16;
   
   /** The delay between mouse enter widget and draw tooltip, ms. */
   private static final int DELAY_MS = 100;
   
   /** The time the tooltip is shown without any action, ms. */
   private static final int DURATION_MS = 5000;
   
   /** Tooltips enabled by default for GUI. */
   private static boolean sessionTooltips = false;

   /** Screen driver. */
   private GuiDriver gd;
   
   /** Font to draw the tooltip. */
   private FontDetails<?> fntToolTip;

   /** The referecnce to widget the tooltip is belonging to. */
   private AbstractWidget<GuiOutputManager> owner = null;

   /** Tooltip background color. */
   private ColorRgb toolTipFgColor;
   
   /** Tooltip background color. */
   private ColorRgb toolTipBgColor;
   
   /** Flag indicating the tooltip is on the screen. */
   private boolean toolTipToHide = false;
   
   /** Starting position to draw the tooltip */
   private NativePoint position;
   
   /**
    * This constructor creates a tooltip object for given screen driver.
    * 
    * @param   wid
    *          The widget's reference for tooltip.
    * @param   fnt
    *          The font to use to draw the tooltip.
    */
   public ToolTip(Widget<GuiOutputManager> wid, FontDetails<?> fnt) 
   {
      // initialize tooltip internals
      // screen driver
      gd = (GuiDriver) OutputManager.getDriver();
      
      // font to render text
      fntToolTip = fnt;
      
      // widget owning the tooltip
      owner = (AbstractWidget) wid;
      
      // tooltip colors
      toolTipFgColor = gd.getSysColor(SysColor.COLOR_BTNTEXT);
      toolTipBgColor = gd.getSysColor(SysColor.COLOR_TOOLTIPBK);
   }
   
   /**
    * Main tooltip life cycle.
    */
   public void run()
   {
      // initial delay before show
      try
      {
         Thread.sleep(DELAY_MS);
      }
      catch (InterruptedException ie)
      {
         // tooltip engine interrupted before text shown
         return;
      }

      // draw the text
      show(position);
      
      // wait the tooltip duration time
      try
      {
         Thread.sleep(DURATION_MS);
      }
      catch (InterruptedException ie)
      {
         // waiting was externally interrupted
      }
      finally
      {
         // hiding the text
         hide();
      }
   }
   
   /**
    * Draw widget tooltip help string worker.
    * 
    * @param   orig
    *          Tooltip origin.
    * @param   text
    *          Tooltip text.
    */
   private void drawInt(NativePoint orig, String text)
   {
      // calculate required rectangle to include the text
      int w = gd.getTextWidth(text, fntToolTip) + 4;
      int h = fntToolTip.pointSize + 4;
      
      // draw immediately
      gd.startBatch();

      // tooltip frame      
      gd.setColor(toolTipFgColor);
      gd.drawRect(orig.x, orig.y, w, h);
      // toltip background
      gd.setColor(toolTipBgColor);
      gd.fillRect(orig.x + 1, orig.y + 1, w - 2, h - 2);

      // tooltip text
      gd.setGuiFont(fntToolTip);
      gd.setFontStyle(fntToolTip.style);
      gd.setColor(toolTipFgColor);
      gd.drawString(text, orig.x + 2, orig.y + h - 3);

      // draw immediately
      gd.endBatch();
   }
   
   /**
    * Setting new position for tooltip drawing.
    * 
    * @param   orig
    *          Tooltip origin.
    */
   public void setOrigin(NativePoint orig)
   {
      position = orig;
   }
   
   /**
    * Checks if the given tooltip engine should be activated in a curren conditions.  This depends
    * on the session wide TOOLTIPS boolean attribute and on the cuttent text value for given
    * widget.  If TOOLTIPS attribute is <code>false</code> or text does not exist, empty or
    * unknow value the tooltip will not be drawn. 
    *
    * @param   tooltipTxt
    *          Tooltip string to check.
    *
    * @return  <code>true</code> if the tooltip needs to be used,
    *          <code>false</code> otherwise.
    */
   public static boolean isActive(String tooltipTxt)
   {
      // first check the SESSION:TOOLTIPS attribute value
      // second check the tooltip string itself
      // null string or empty string both disable the tooltip engine
      return sessionTooltips && tooltipTxt != null && tooltipTxt.length() > 0;
   }

   /**
    * Sets the SESSION:TOOLTIPS attribute for this session.
    *
    * @param    tooltips
    *           The new value for session related TOOLTIPS attribute.
    */
   public static void setSessionTooltips(boolean tooltips)
   {
      // if nothing changed - do nothing
      if (sessionTooltips == tooltips)
      {
         return;
      }
      
      // accept new value
      sessionTooltips = tooltips;
   }
   
   /**
    * Accesses the SESSION:TOOLTIPS attribute for this session.
    *
    * @return   <code>true</code> if tooltips are available for this session
    *           <code>false</code> otherwise.
    */
   public static boolean getSessionTooltips()
   {
      return sessionTooltips;
   }
   
   /**
    * Draw widget tooltip help string at the specified position.
    * 
    * @param   orig
    *          Tooltip origin.
    */
   private void show(NativePoint orig)
   {
      // simple validity check
      if (owner == null)
      {
         return;
      }
      
      // get the current tooltip text, it can be different from call to call
      final String toolTip = UiUtils.getConfig(owner, BaseConfig.class).tooltip;
      
      if (isActive(toolTip))
      {
         // mark we have something to be hidden further
         toolTipToHide = true;
         // draw the tooltip itself
         gd.draw(orig, null, new Runnable()
         {
                 
            @Override
            public void run()
            {
               // inside this call widget origin is always (1,1)
               NativePoint po = new NativePoint(0, ORIGIN_SHIFT);
                  
               // draw the tooltip itself
               drawInt(po, toolTip);
            }
         
         });
      }
   }
   
   /**
    * Hiding the widget tooltip string if one exists.
    */
   private void hide()
   {
      if (toolTipToHide)
      {
         toolTipToHide = false;
         
         ThinClient.getInstance().eventDrawingBracket(owner, false, true, new Runnable()
         {
            @Override
            public void run()
            {
               owner.repaint();
            }
         });
      }
   }
}
