/*
** Module   : SwingGuiClient.java
** Abstract : GUI-specific OS-level window component for the AWT/Swing client
**
** Copyright (c) 2014-2015, Golden Code Development Corporation.
** ALL RIGHTS RESERVED. Use is subject to license terms.
**
**           Golden Code Development Corporation
**                      CONFIDENTIAL
**
** -#- -I- --Date-- ---------------------------------Description---------------------------------
** 001 MAG 20140508 First version.
** 002 MAG 20140925 Hide constructor.
** 003 CA  20141021 Do not allow GUI windows to be closed via OS (i.e. ALT-F4).
** 004 CA  20141029 Each client needs to receive the swing keyboard reader implementation in 
**                  use.  It's the callers responsibility to ensure that the same instance is used
**                  for all simulators, if the client is ran in a multi-window mode (i.e. GUI).
** 005 CA  20141119 Refactored drawing code to enable multiple OS windows, for GUI support.  Misc
**                  changes/fixes related to drawing on multiple windows.
** 006 CA  20150415 Moved the default title to the WindowGuiImpl class.
** 007 GES 20150504 Changed imports/member names to match package moves and class renames.
**                  Renamed this class to better describe the specific nature.
*/

package com.goldencode.p2j.ui.client.gui.driver.swing;

import java.lang.reflect.*;

import javax.swing.*;

import com.goldencode.p2j.ui.client.gui.*;
import com.goldencode.p2j.ui.client.gui.driver.*;
import com.goldencode.p2j.ui.client.driver.swing.*;

/** 
 * Main GUI client class which instantiates the primary frame window.
 */
@SuppressWarnings("serial")
public class SwingGuiClient 
extends AbstractClient
{
   /** Window emulator instance. */
   final private SwingEmulatedWindow sew;

   /**
    * Construct the new frame by instantiating the {@link SwingEmulatedWindow} as the
    * content pane and initializing it using the given <code>options</code>.
    * 
    * @param    keyReader
    *           The {@link SwingKeyboardReader key reader} which listens for incoming keys.
    */
   private SwingGuiClient(SwingKeyboardReader keyReader)
   {
      super(WindowGuiImpl.DEFAULT_TITLE);
      
      // common initialization
      sew = new SwingEmulatedWindow(this, keyReader);
      init(sew.getContentPane(), true);
   }

   /**
    * Initialize the application frame by creating a new instance of this class
    * and of the content pane which is an instance of {@link SwingEmulatedWindow}.
    * 
    * @param    keyReader
    *           The {@link SwingKeyboardReader key reader} which listens for incoming keys.
    *           
    * @return   The simulator instance.
    */                                                                     
   public static EmulatedWindowState load(final SwingKeyboardReader keyReader) 
   throws InterruptedException,
          InvocationTargetException
   {
      // lame hack because of Java flaw
      final SwingGuiClient[] client = new SwingGuiClient[1];

      // instantiate the application's main frame
      SwingUtilities.invokeAndWait(new Runnable()
      {
         public void run()
         {
            client[0] = new SwingGuiClient(keyReader);
         }
      });

      return client[0].sew;
   }
   
   /**
    * Initialize.  GUI windows are not allowed to be closed via OS (i.e. ALT-F4).
    * 
    * @param   pane
    *          Content pane.
    * @param   resizable
    *          Resizable attribute.
    */
   @Override
   protected void init(JComponent pane, boolean resizable)
   {
      super.init(pane, resizable);

      setDefaultCloseOperation(JFrame.DO_NOTHING_ON_CLOSE);
                                                              
      // by default, the simulator is hidden
      sew.setWindowVisible(false);
   }
}
